@@ -6,7 +6,7 @@ from pathlib import Path
 import random
 import re
 from shutil import rmtree
-from typing import Union
+from typing import List, Union
 
 import anys
 import click
@@ -24,9 +24,10 @@ from ..consts import (
     dandiset_identifier_regex,
     dandiset_metadata_file,
 )
-from ..dandiapi import DandiAPIClient, Version
+from ..dandiapi import DandiAPIClient, RemoteAsset, RemoteZarrAsset, Version
 from ..download import download
 from ..exceptions import NotFoundError, SchemaVersionError
+from ..files import GenericAsset, dandi_file
 from ..utils import list_paths
 
 
@@ -49,6 +50,7 @@ def test_publish_and_manipulate(new_dandiset: SampleDandiset, tmp_path: Path) ->
     assert str(d) == f"DANDI-API-LOCAL-DOCKER-TESTS:{dandiset_id}/draft"
     (dspath / "subdir").mkdir()
     (dspath / "subdir" / "file.txt").write_text("This is test text.\n")
+    (dspath / "subdir" / "doomed.txt").write_text("This will be deleted.\n")
     new_dandiset.upload(allow_any_path=True)
 
     d.wait_until_valid()
@@ -61,21 +63,11 @@ def test_publish_and_manipulate(new_dandiset: SampleDandiset, tmp_path: Path) ->
     dandiset_yaml = tmp_path / dandiset_id / dandiset_metadata_file
     file_in_version = tmp_path / dandiset_id / "subdir" / "file.txt"
 
-    download(dv.version_api_url, tmp_path)
-    assert list_paths(tmp_path) == [dandiset_yaml, file_in_version]
-    assert file_in_version.read_text() == "This is test text.\n"
-
     (dspath / "subdir" / "file.txt").write_text("This is different text.\n")
-    new_dandiset.upload(allow_any_path=True)
-    rmtree(tmp_path / dandiset_id)
-    download(dv.version_api_url, tmp_path)
-    assert list_paths(tmp_path) == [dandiset_yaml, file_in_version]
-    assert file_in_version.read_text() == "This is test text.\n"
-
     (dspath / "subdir" / "file2.txt").write_text("This is more text.\n")
     new_dandiset.upload(allow_any_path=True)
+    d.get_asset_by_path("subdir/doomed.txt").delete()
 
-    rmtree(tmp_path / dandiset_id)
     download(d.version_api_url, tmp_path)
     assert list_paths(tmp_path) == [
         dandiset_yaml,
@@ -87,23 +79,15 @@ def test_publish_and_manipulate(new_dandiset: SampleDandiset, tmp_path: Path) ->
 
     rmtree(tmp_path / dandiset_id)
     download(dv.version_api_url, tmp_path)
-    assert list_paths(tmp_path) == [dandiset_yaml, file_in_version]
-    assert file_in_version.read_text() == "This is test text.\n"
-
-    d.get_asset_by_path("subdir/file.txt").delete()
-
-    rmtree(tmp_path / dandiset_id)
-    download(d.version_api_url, tmp_path)
     assert list_paths(tmp_path) == [
         dandiset_yaml,
-        file_in_version.with_name("file2.txt"),
+        file_in_version.with_name("doomed.txt"),
+        file_in_version,
     ]
-    assert file_in_version.with_name("file2.txt").read_text() == "This is more text.\n"
-
-    rmtree(tmp_path / dandiset_id)
-    download(dv.version_api_url, tmp_path)
-    assert list_paths(tmp_path) == [dandiset_yaml, file_in_version]
     assert file_in_version.read_text() == "This is test text.\n"
+    assert (
+        file_in_version.with_name("doomed.txt").read_text() == "This will be deleted.\n"
+    )
 
 
 def test_get_asset_metadata(new_dandiset: SampleDandiset, simple1_nwb: str) -> None:
@@ -620,3 +604,45 @@ def test_get_assets_with_path_prefix(text_dandiset: SampleDandiset) -> None:
             "subdir", order="-path"
         )
     ] == ["subdir2/coconut.txt", "subdir2/banana.txt", "subdir1/apple.txt"]
+
+
+def test_empty_zarr_iterfiles(new_dandiset: SampleDandiset) -> None:
+    client = new_dandiset.client
+    r = client.post(
+        "/zarr/", json={"name": "empty.zarr", "dandiset": new_dandiset.dandiset_id}
+    )
+    zarr_id = r["zarr_id"]
+    r = client.post(
+        f"{new_dandiset.dandiset.version_api_path}assets/",
+        json={"metadata": {"path": "empty.zarr"}, "zarr_id": zarr_id},
+    )
+    a = RemoteAsset.from_data(new_dandiset.dandiset, r)
+    assert isinstance(a, RemoteZarrAsset)
+    assert list(a.iterfiles()) == []
+
+
+def test_get_many_pages_of_assets(
+    mocker: MockerFixture, new_dandiset: SampleDandiset
+) -> None:
+    new_dandiset.client.page_size = 4
+    get_spy = mocker.spy(new_dandiset.client, "get")
+    paths: List[str] = []
+    for i in range(26):
+        p = new_dandiset.dspath / f"{i:04}.txt"
+        paths.append(p.name)
+        p.write_text(f"File #{i}\n")
+        df = dandi_file(p, new_dandiset.dspath)
+        assert isinstance(df, GenericAsset)
+        df.upload(new_dandiset.dandiset, {"description": f"File #{i}"})
+    assert [
+        asset.path for asset in new_dandiset.dandiset.get_assets(order="path")
+    ] == paths
+    assert get_spy.call_count == 7
+    pth = f"{new_dandiset.dandiset.version_api_path}assets/"
+    get_spy.assert_any_call(
+        pth, params={"order": "path", "page_size": 4}, json_resp=False
+    )
+    for n in range(2, 8):
+        get_spy.assert_any_call(
+            pth, params={"order": "path", "page_size": 4, "page": n}
+        )
